@@ -1,7 +1,5 @@
 #!/bin/sh
 #
-# $Id: autogen.sh 2222 2007-11-01 21:47:58Z des $
-#
 
 warn() {
 	echo "WARNING: $@" 1>&2
@@ -40,7 +38,7 @@ fi
 
 set -ex
 
-aclocal
+aclocal -I m4
 $LIBTOOLIZE --copy --force
 autoheader
 automake --add-missing --copy --foreign
