@@ -1,5 +1,5 @@
 /*-
- * Copyright (c) 2008-2009 Linpro AS
+ * Copyright (c) 2008-2010 Varnish Software AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -35,22 +35,13 @@
 
 #include "config.h"
 
-#include "svnid.h"
-SVNID("$Id: cache_lck.c 4599 2010-03-01 11:20:21Z phk $")
-
 #include <stdio.h>
 
-#include <pthread.h>
-#ifdef HAVE_PTHREAD_NP_H
-#include <pthread_np.h>
-#endif
 #include <stdlib.h>
 
-#include "shmlog.h"
 #include "cache.h"
 
 /*The constability of lck depends on platform pthreads implementation */
-/*lint -save -esym(818,lck) */
 
 struct ilck {
 	unsigned		magic;
@@ -60,6 +51,7 @@ struct ilck {
 	pthread_t		owner;
 	VTAILQ_ENTRY(ilck)	list;
 	const char		*w;
+	struct VSC_C_lck	*stat;
 };
 
 static VTAILQ_HEAD(, ilck)	ilck_head =
@@ -67,7 +59,7 @@ static VTAILQ_HEAD(, ilck)	ilck_head =
 
 static pthread_mutex_t		lck_mtx;
 
-void
+void __match_proto__()
 Lck__Lock(struct lock *lck, const char *p, const char *f, int l)
 {
 	struct ilck *ilck;
@@ -77,6 +69,7 @@ Lck__Lock(struct lock *lck, const char *p, const char *f, int l)
 	if (!(params->diag_bitmap & 0x18)) {
 		AZ(pthread_mutex_lock(&ilck->mtx));
 		AZ(ilck->held);
+		ilck->stat->locks++;
 		ilck->owner = pthread_self();
 		ilck->held = 1;
 		return;
@@ -84,17 +77,21 @@ Lck__Lock(struct lock *lck, const char *p, const char *f, int l)
 	r = pthread_mutex_trylock(&ilck->mtx);
 	assert(r == 0 || r == EBUSY);
 	if (r) {
-		VSL(SLT_Debug, 0, "MTX_CONTEST(%s,%s,%d,%s)", p, f, l, ilck->w);
+		ilck->stat->colls++;
+		if (params->diag_bitmap & 0x8)
+			VSL(SLT_Debug, 0, "MTX_CONTEST(%s,%s,%d,%s)",
+			    p, f, l, ilck->w);
 		AZ(pthread_mutex_lock(&ilck->mtx));
 	} else if (params->diag_bitmap & 0x8) {
 		VSL(SLT_Debug, 0, "MTX_LOCK(%s,%s,%d,%s)", p, f, l, ilck->w);
 	}
 	AZ(ilck->held);
+	ilck->stat->locks++;
 	ilck->owner = pthread_self();
 	ilck->held = 1;
 }
 
-void
+void __match_proto__()
 Lck__Unlock(struct lock *lck, const char *p, const char *f, int l)
 {
 	struct ilck *ilck;
@@ -108,7 +105,7 @@ Lck__Unlock(struct lock *lck, const char *p, const char *f, int l)
 		VSL(SLT_Debug, 0, "MTX_UNLOCK(%s,%s,%d,%s)", p, f, l, ilck->w);
 }
 
-int
+int __match_proto__()
 Lck__Trylock(struct lock *lck, const char *p, const char *f, int l)
 {
 	struct ilck *ilck;
@@ -142,7 +139,7 @@ Lck__Assert(const struct lock *lck, int held)
 		    !pthread_equal(ilck->owner, pthread_self()));
 }
 
-void
+void __match_proto__()
 Lck_CondWait(pthread_cond_t *cond, struct lock *lck)
 {
 	struct ilck *ilck;
@@ -158,14 +155,17 @@ Lck_CondWait(pthread_cond_t *cond, struct lock *lck)
 }
 
 void
-Lck__New(struct lock *lck, const char *w)
+Lck__New(struct lock *lck, struct VSC_C_lck *st, const char *w)
 {
 	struct ilck *ilck;
 
+	AN(st);
 	AZ(lck->priv);
 	ALLOC_OBJ(ilck, ILCK_MAGIC);
 	AN(ilck);
 	ilck->w = w;
+	ilck->stat = st;
+	ilck->stat->creat++;
 	AZ(pthread_mutex_init(&ilck->mtx, NULL));
 	AZ(pthread_mutex_lock(&lck_mtx));
 	VTAILQ_INSERT_TAIL(&ilck_head, ilck, list);
@@ -179,6 +179,7 @@ Lck_Delete(struct lock *lck)
 	struct ilck *ilck;
 
 	CAST_OBJ_NOTNULL(ilck, lck->priv, ILCK_MAGIC);
+	ilck->stat->destroy++;
 	lck->priv = NULL;
 	AZ(pthread_mutex_lock(&lck_mtx));
 	VTAILQ_REMOVE(&ilck_head, ilck, list);
@@ -187,12 +188,18 @@ Lck_Delete(struct lock *lck)
 	FREE_OBJ(ilck);
 }
 
+#define LOCK(nam) struct VSC_C_lck *lck_##nam;
+#include "locks.h"
+#undef LOCK
 
 void
 LCK_Init(void)
 {
 
 	AZ(pthread_mutex_init(&lck_mtx, NULL));
+#define LOCK(nam)						\
+	lck_##nam = VSM_Alloc(sizeof(struct VSC_C_lck),		\
+	   VSC_CLASS, VSC_TYPE_LCK, #nam);
+#include "locks.h"
+#undef LOCK
 }
-
-/*lint -restore */
