@@ -42,92 +42,11 @@
 /*--------------------------------------------------------------------*/
 
 static void
-res_do_304(struct sess *sp)
-{
-	char lm[64];
-	char *p;
-
-	http_ClrHeader(sp->wrk->resp);
-	sp->wrk->resp->logtag = HTTP_Tx;
-	http_SetResp(sp->wrk->resp, "HTTP/1.1", 304, "Not Modified");
-	TIM_format(sp->t_req, lm);
-	http_PrintfHeader(sp->wrk, sp->fd, sp->wrk->resp, "Date: %s", lm);
-	http_SetHeader(sp->wrk, sp->fd, sp->wrk->resp, "Via: 1.1 varnish");
-	http_PrintfHeader(sp->wrk, sp->fd, sp->wrk->resp,
-	    "X-Varnish: %u", sp->xid);
-	if (sp->obj->last_modified) {
-		TIM_format(sp->obj->last_modified, lm);
-		http_PrintfHeader(sp->wrk, sp->fd, sp->wrk->resp,
-		    "Last-Modified: %s", lm);
-	}
-
-	/* http://www.w3.org/Protocols/rfc2616/rfc2616-sec10.html#sec10.3.5 */
-	if (http_GetHdr(sp->obj->http, H_Cache_Control, &p))
-		http_PrintfHeader(sp->wrk, sp->fd, sp->wrk->resp,
-		    "Cache-Control: %s", p);
-	if (http_GetHdr(sp->obj->http, H_Content_Location, &p))
-		http_PrintfHeader(sp->wrk, sp->fd, sp->wrk->resp,
-		    "Content-Location: %s", p);
-	if (http_GetHdr(sp->obj->http, H_ETag, &p))
-		http_PrintfHeader(sp->wrk, sp->fd, sp->wrk->resp,
-		    "ETag: %s", p);
-	if (http_GetHdr(sp->obj->http, H_Expires, &p))
-		http_PrintfHeader(sp->wrk, sp->fd, sp->wrk->resp,
-		    "Expires: %s", p);
-	if (http_GetHdr(sp->obj->http, H_Vary, &p))
-		http_PrintfHeader(sp->wrk, sp->fd, sp->wrk->resp,
-		    "Vary: %s", p);
-
-	http_PrintfHeader(sp->wrk, sp->fd, sp->wrk->resp, "Connection: %s",
-	    sp->doclose ? "close" : "keep-alive");
-	sp->wantbody = 0;
-}
-
-/*--------------------------------------------------------------------*/
-
-static int
-res_do_conds(struct sess *sp)
-{
-	char *p, *e;
-	double ims;
-	int do_cond = 0;
-
-	/* RFC 2616 13.3.4 states we need to match both ETag
-	   and If-Modified-Since if present*/
-
-	if (http_GetHdr(sp->http, H_If_Modified_Since, &p) ) {
-		if (!sp->obj->last_modified)
-			return (0);
-		ims = TIM_parse(p);
-		if (ims > sp->t_req)	/* [RFC2616 14.25] */
-			return (0);
-		if (sp->obj->last_modified > ims)
-			return (0);
-		do_cond = 1;
-	}
-
-	if (http_GetHdr(sp->http, H_If_None_Match, &p) &&
-	    http_GetHdr(sp->obj->http, H_ETag, &e)) {
-		if (strcmp(p,e) != 0)
-			return (0);
-		do_cond = 1;
-	}
-
-	if (do_cond == 1) {
-		res_do_304(sp);
-		return (1);
-	}
-	return (0);
-}
-
-/*--------------------------------------------------------------------*/
-
-static void
-res_dorange(struct sess *sp, const char *r, ssize_t *plow, ssize_t *phigh)
+res_dorange(const struct sess *sp, const char *r, ssize_t *plow, ssize_t *phigh)
 {
 	ssize_t low, high, has_low;
 
-	(void)sp;
+	assert(sp->obj->response == 200);
 	if (strncmp(r, "bytes=", 6))
 		return;
 	r += 6;
@@ -189,14 +108,12 @@ res_dorange(struct sess *sp, const char *r, ssize_t *plow, ssize_t *phigh)
 /*--------------------------------------------------------------------*/
 
 void
-RES_BuildHttp(struct sess *sp)
+RES_BuildHttp(const struct sess *sp)
 {
 	char time_str[30];
 
 	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
 
-	if (sp->obj->response == 200 && sp->http->conds && res_do_conds(sp))
-		return;
 
 	http_ClrHeader(sp->wrk->resp);
 	sp->wrk->resp->logtag = HTTP_Tx;
@@ -226,7 +143,7 @@ RES_BuildHttp(struct sess *sp)
 		http_PrintfHeader(sp->wrk, sp->fd, sp->wrk->resp,
 		    "X-Varnish: %u", sp->xid);
 	http_PrintfHeader(sp->wrk, sp->fd, sp->wrk->resp, "Age: %.0f",
-	    sp->obj->age + sp->t_resp - sp->obj->entered);
+	    sp->obj->exp.age + sp->t_resp - sp->obj->exp.entered);
 	http_SetHeader(sp->wrk, sp->fd, sp->wrk->resp, "Via: 1.1 varnish");
 	http_PrintfHeader(sp->wrk, sp->fd, sp->wrk->resp, "Connection: %s",
 	    sp->doclose ? "close" : "keep-alive");
@@ -258,13 +175,13 @@ res_WriteGunzipObj(struct sess *sp)
 		CHECK_OBJ_NOTNULL(st, STORAGE_MAGIC);
 		u += st->len;
 
-		sp->acct_tmp.bodybytes += st->len;	/* XXX ? */
 		VSC_C_main->n_objwrite++;
 
 		i = VGZ_WrwGunzip(sp, vg,
 		    st->ptr, st->len,
 		    obuf, sizeof obuf, &obufl);
 		/* XXX: error check */
+		(void)i;
 	}
 	if (obufl) {
 		(void)WRW_Write(sp->wrk, obuf, obufl);
@@ -277,7 +194,7 @@ res_WriteGunzipObj(struct sess *sp)
 /*--------------------------------------------------------------------*/
 
 static void
-res_WriteDirObj(struct sess *sp, ssize_t low, ssize_t high)
+res_WriteDirObj(const struct sess *sp, ssize_t low, ssize_t high)
 {
 	ssize_t u = 0;
 	size_t ptr, off, len;
@@ -309,7 +226,7 @@ res_WriteDirObj(struct sess *sp, ssize_t low, ssize_t high)
 
 		ptr += len;
 
-		sp->acct_tmp.bodybytes += len;
+		sp->wrk->acct_tmp.bodybytes += len;
 #ifdef SENDFILE_WORKS
 		/*
 		 * XXX: the overhead of setting up sendfile is not
@@ -330,7 +247,10 @@ res_WriteDirObj(struct sess *sp, ssize_t low, ssize_t high)
 	assert(u == sp->obj->len);
 }
 
-/*--------------------------------------------------------------------*/
+/*--------------------------------------------------------------------
+ * Deliver an object.
+ * Attempt optimizations like 304 and 206 here.
+ */
 
 void
 RES_WriteObj(struct sess *sp)
@@ -342,14 +262,27 @@ RES_WriteObj(struct sess *sp)
 
 	WRW_Reserve(sp->wrk, &sp->fd);
 
+	if (sp->obj->response == 200 &&
+	    sp->http->conds &&
+	    RFC2616_Do_Cond(sp)) {
+		sp->wantbody = 0;
+		http_SetResp(sp->wrk->resp, "HTTP/1.1", 304, "Not Modified");
+		http_Unset(sp->wrk->resp, H_Content_Length);
+		http_Unset(sp->wrk->resp, H_Transfer_Encoding);
+	}
+
 	/*
 	 * If nothing special planned, we can attempt Range support
 	 */
 	low = 0;
 	high = sp->obj->len - 1;
-	if (!(sp->wrk->res_mode & (RES_ESI|RES_ESI_CHILD|RES_GUNZIP)) &&
-	    params->http_range_support && sp->obj->response == 200 &&
-	    sp->wantbody && http_GetHdr(sp->http, H_Range, &r))
+	if (
+	    sp->wantbody &&
+	    (sp->wrk->res_mode & RES_LEN) &&
+	    !(sp->wrk->res_mode & (RES_ESI|RES_ESI_CHILD|RES_GUNZIP)) &&
+	    params->http_range_support &&
+	    sp->obj->response == 200 &&
+	    http_GetHdr(sp->http, H_Range, &r))
 		res_dorange(sp, r, &low, &high);
 
 	/*
@@ -362,7 +295,7 @@ RES_WriteObj(struct sess *sp)
 	 * Send HTTP protocol header, unless interior ESI object
 	 */
 	if (!(sp->wrk->res_mode & RES_ESI_CHILD))
-		sp->acct_tmp.hdrbytes +=
+		sp->wrk->acct_tmp.hdrbytes +=
 		    http_Write(sp->wrk, sp->wrk->resp, 1);
 
 	if (!sp->wantbody)
@@ -372,7 +305,7 @@ RES_WriteObj(struct sess *sp)
 		WRW_Chunked(sp->wrk);
 
 	if (!sp->wantbody) {
-		/* This was a HEAD request */
+		/* This was a HEAD or conditional request */
 	} else if (sp->obj->len == 0) {
 		/* Nothing to do here */
 	} else if (sp->wrk->res_mode & RES_ESI) {
@@ -418,7 +351,7 @@ RES_StreamStart(struct sess *sp)
 		http_PrintfHeader(sp->wrk, sp->fd, sp->wrk->resp,
 		    "Content-Length: %s", sp->wrk->h_content_length);
 
-	sp->acct_tmp.hdrbytes +=
+	sp->wrk->acct_tmp.hdrbytes +=
 	    http_Write(sp->wrk, sp->wrk->resp, 1);
 
 	if (sp->wrk->res_mode & RES_CHUNKED)
