@@ -145,7 +145,7 @@ usage(void)
 	fprintf(stderr, FMT, "", "  shl: space for SHL records [80m]");
 	fprintf(stderr, FMT, "", "  free: space for other allocations [1m]");
 	fprintf(stderr, FMT, "", "  fill: prefill new file [+]");
-	fprintf(stderr, FMT, "-M address:port", "CLI-master to connect to.");
+	fprintf(stderr, FMT, "-M address:port", "Reverse CLI destination.");
 	fprintf(stderr, FMT, "-n dir", "varnishd working directory");
 	fprintf(stderr, FMT, "-P file", "PID file");
 	fprintf(stderr, FMT, "-p param=value", "set parameter");
@@ -418,6 +418,9 @@ main(int argc, char * const *argv)
 		MCF_ParamSet(cli, "http_resp_size", "8192");
 		cli_check(cli);
 
+		MCF_ParamSet(cli, "http_req_size", "12288");
+		cli_check(cli);
+
 		MCF_ParamSet(cli, "thread_pool_stack", "32bit");
 		cli_check(cli);
 
@@ -500,12 +503,10 @@ main(int argc, char * const *argv)
 			VCS_Message("varnishd");
 			exit(0);
 		case 'x':
-#ifdef DIAGNOSTICS
 			if (!strcmp(optarg, "dumprst")) {
 				MCF_DumpRst();
 				exit (0);
 			}
-#endif /* DIAGNOSTICS */
 			usage();
 			break;
 		case 'w':
