@@ -31,7 +31,7 @@
  * Obtain log data from the shared memory log, order it by session ID, and
  * display it in Apache / NCSA combined log format:
  *
- *	%h %l %u %t "%r" %>s %b "%{Referer}i" "%{User-agent}i"
+ *	%h %l %u %t "%r" %s %b "%{Referer}i" "%{User-agent}i"
  *
  * where the fields are defined as follows:
  *
@@ -54,8 +54,7 @@
  *	%q		Query string
  *	%H		Protocol version
  *
- * TODO:	- Make it possible to grab any request header
- *		- Maybe rotate/compress log
+ * TODO:		- Maybe rotate/compress log
  */
 
 #include "config.h"
@@ -72,6 +71,7 @@
 
 #include "vsb.h"
 #include "vpf.h"
+#include "vqueue.h"
 
 #include "libvarnish.h"
 #include "vsl.h"
@@ -81,14 +81,16 @@
 
 static volatile sig_atomic_t reopen;
 
+struct hdr {
+	char *key;
+	char *value;
+	VTAILQ_ENTRY(hdr) list;
+};
+
 static struct logline {
 	char *df_H;			/* %H, Protocol version */
-	char *df_Host;			/* %{Host}i */
-	char *df_Referer;		/* %{Referer}i */
 	char *df_U;			/* %U, URL path */
 	char *df_q;			/* %q, query string */
-	char *df_User_agent;		/* %{User-agent}i */
-	char *df_X_Forwarded_For;	/* %{X-Forwarded-For}i */
 	char *df_b;			/* %b, Bytes */
 	char *df_h;			/* %h (host name / IP adress)*/
 	char *df_m;			/* %m, Request method*/
@@ -101,6 +103,8 @@ static struct logline {
 	int active;			/* Is log line in an active trans */
 	int complete;			/* Is log line complete */
 	uint64_t bitmap;		/* Bitmap for regex matches */
+	VTAILQ_HEAD(, hdr) req_headers; /* Request headers */
+	VTAILQ_HEAD(, hdr) resp_headers; /* Response headers */
 } **ll;
 
 struct VSM_data *vd;
@@ -186,23 +190,58 @@ trimline(const char *str, const char *end)
 	return (p);
 }
 
+static char *
+req_header(struct logline *l, const char *name)
+{
+	struct hdr *h;
+	VTAILQ_FOREACH(h, &l->req_headers, list) {
+		if (strcasecmp(h->key, name) == 0) {
+			return h->value;
+			break;
+		}
+	}
+	return NULL;
+}
+
+static char *
+resp_header(struct logline *l, const char *name)
+{
+	struct hdr *h;
+	VTAILQ_FOREACH(h, &l->resp_headers, list) {
+		if (strcasecmp(h->key, name) == 0) {
+			return h->value;
+			break;
+		}
+	}
+	return NULL;
+}
+
 static void
 clean_logline(struct logline *lp)
 {
+	struct hdr *h, *h2;
 #define freez(x) do { if (x) free(x); x = NULL; } while (0);
 	freez(lp->df_H);
-	freez(lp->df_Host);
-	freez(lp->df_Referer);
 	freez(lp->df_U);
 	freez(lp->df_q);
-	freez(lp->df_User_agent);
-	freez(lp->df_X_Forwarded_For);
 	freez(lp->df_b);
 	freez(lp->df_h);
 	freez(lp->df_m);
 	freez(lp->df_s);
 	freez(lp->df_u);
 	freez(lp->df_ttfb);
+	VTAILQ_FOREACH_SAFE(h, &lp->req_headers, list, h2) {
+		VTAILQ_REMOVE(&lp->req_headers, h, list);
+		freez(h->key);
+		freez(h->value);
+		freez(h);
+	}
+	VTAILQ_FOREACH_SAFE(h, &lp->resp_headers, list, h2) {
+		VTAILQ_REMOVE(&lp->resp_headers, h, list);
+		freez(h->key);
+		freez(h->value);
+		freez(h);
+	}
 #undef freez
 	memset(lp, 0, sizeof *lp);
 }
@@ -211,7 +250,7 @@ static int
 collect_backend(struct logline *lp, enum VSL_tag_e tag, unsigned spec,
     const char *ptr, unsigned len)
 {
-	const char *end, *next;
+	const char *end, *next, *split;
 
 	assert(spec & VSL_S_BACKEND);
 	end = ptr + len;
@@ -293,17 +332,23 @@ collect_backend(struct logline *lp, enum VSL_tag_e tag, unsigned spec,
 	case SLT_TxHeader:
 		if (!lp->active)
 			break;
-		if (isprefix(ptr, "user-agent:", end, &next))
-			lp->df_User_agent = trimline(next, end);
-		else if (isprefix(ptr, "referer:", end, &next))
-			lp->df_Referer = trimline(next, end);
-		else if (isprefix(ptr, "authorization:", end, &next) &&
-		    isprefix(next, "basic", end, &next))
+		split = strchr(ptr, ':');
+		if (split == NULL)
+			break;
+		if (isprefix(ptr, "authorization:", end, &next) &&
+		    isprefix(next, "basic", end, &next)) {
 			lp->df_u = trimline(next, end);
-		else if (isprefix(ptr, "x-forwarded-for:", end, &next))
-			lp->df_X_Forwarded_For = trimline(next, end);
-		else if (isprefix(ptr, "host:", end, &next))
-			lp->df_Host = trimline(next, end);
+		} else {
+			struct hdr *h;
+			size_t l;
+			h = malloc(sizeof(struct hdr));
+			AN(h);
+			AN(split);
+			l = strlen(split);
+			h->key = trimline(ptr, split-1);
+			h->value = trimline(split+1, split+l-1);
+			VTAILQ_INSERT_HEAD(&lp->req_headers, h, list);
+		}
 		break;
 
 	case SLT_BackendReuse:
@@ -325,7 +370,7 @@ static int
 collect_client(struct logline *lp, enum VSL_tag_e tag, unsigned spec,
     const char *ptr, unsigned len)
 {
-	const char *end, *next;
+	const char *end, *next, *split;
 	long l;
 	time_t t;
 
@@ -391,25 +436,29 @@ collect_client(struct logline *lp, enum VSL_tag_e tag, unsigned spec,
 			lp->df_s = trimline(ptr, end);
 		break;
 
+	case SLT_TxHeader:
 	case SLT_RxHeader:
 		if (!lp->active)
 			break;
-		if (isprefix(ptr, "user-agent:", end, &next)) {
-			free(lp->df_User_agent);
-			lp->df_User_agent = trimline(next, end);
-		} else if (isprefix(ptr, "referer:", end, &next)) {
-			free(lp->df_Referer);
-			lp->df_Referer = trimline(next, end);
-		} else if (isprefix(ptr, "authorization:", end, &next) &&
-			   isprefix(next, "basic", end, &next)) {
+		split = strchr(ptr, ':');
+		if (split == NULL)
+			break;
+		if (tag == SLT_RxHeader &&
+		    isprefix(ptr, "authorization:", end, &next) &&
+		    isprefix(next, "basic", end, &next)) {
 			free(lp->df_u);
 			lp->df_u = trimline(next, end);
-		} else if (isprefix(ptr, "x-forwarded-for:", end, &next)) {
-			free(lp->df_X_Forwarded_For);
-			lp->df_X_Forwarded_For = trimline(next, end);
-		} else if (isprefix(ptr, "host:", end, &next)) {
-			free(lp->df_Host);
-			lp->df_Host = trimline(next, end);
+		} else {
+			struct hdr *h;
+			h = malloc(sizeof(struct hdr));
+			AN(h);
+			AN(split);
+			h->key = trimline(ptr, split);
+			h->value = trimline(split+1, end);
+			if (tag == SLT_RxHeader)
+				VTAILQ_INSERT_HEAD(&lp->req_headers, h, list);
+			else
+				VTAILQ_INSERT_HEAD(&lp->resp_headers, h, list);
 		}
 		break;
 
@@ -485,6 +534,7 @@ h_ncsa(void *priv, enum VSL_tag_e tag, unsigned fd,
 	FILE *fo = priv;
 	char *q, tbuf[64];
 	const char *p;
+	struct vsb *os;
 
 	if (fd >= nll) {
 		struct logline **newll = ll;
@@ -533,12 +583,12 @@ h_ncsa(void *priv, enum VSL_tag_e tag, unsigned fd,
 	/* We have a complete data set - log a line */
 
 	fo = priv;
+	os = VSB_new_auto();
 
 	for (p = format; *p != '\0'; p++) {
 
-
 		if (*p != '%') {
-			fprintf(fo, "%c", *p);
+			VSB_putc(os, *p);
 			continue;
 		}
 		p++;
@@ -546,29 +596,29 @@ h_ncsa(void *priv, enum VSL_tag_e tag, unsigned fd,
 
 		case 'b':
 			/* %b */
-			fprintf(fo, "%s", lp->df_b ? lp->df_b : "-");
+			VSB_cat(os, lp->df_b ? lp->df_b : "-");
 			break;
 
 		case 'H':
-			fprintf(fo, "%s", lp->df_H);
+			VSB_cat(os, lp->df_H ? lp->df_H : "HTTP/1.0");
 			break;
 
 		case 'h':
 			if (!lp->df_h && spec & VSL_S_BACKEND)
-				fprintf(fo, "127.0.0.1");
+				VSB_cat(os, "127.0.0.1");
 			else
-				fprintf(fo, "%s", lp->df_h ? lp->df_h : "-");
+				VSB_cat(os, lp->df_h ? lp->df_h : "-");
 			break;
 		case 'l':
-			fprintf(fo, "-");
+			VSB_putc(os, '-');
 			break;
 
 		case 'm':
-			fprintf(fo, "%s", lp->df_m);
+			VSB_cat(os, lp->df_m ? lp->df_m : "-");
 			break;
 
 		case 'q':
-			fprintf(fo, "%s", lp->df_q ? lp->df_q : "");
+			VSB_cat(os, lp->df_q ? lp->df_q : "");
 			break;
 
 		case 'r':
@@ -576,30 +626,34 @@ h_ncsa(void *priv, enum VSL_tag_e tag, unsigned fd,
 			 * Fake "%r".  This would be a lot easier if Varnish
 			 * normalized the request URL.
 			 */
-			fprintf(fo, "%s ", lp->df_m);
-			if (lp->df_Host) {
-				if (strncmp(lp->df_Host, "http://", 7) != 0)
-					fprintf(fo, "http://");
-				fprintf(fo, "%s", lp->df_Host);
+			VSB_cat(os, lp->df_m ? lp->df_m : "-");
+			VSB_putc(os, ' ');
+			if (req_header(lp, "Host")) {
+				if (strncmp(req_header(lp, "Host"), "http://", 7) != 0)
+					VSB_cat(os, "http://");
+				VSB_cat(os, req_header(lp, "Host"));
+			} else {
+				VSB_cat(os, "http://localhost");
 			}
-			fprintf(fo, "%s", lp->df_U);
-			fprintf(fo, "%s ", lp->df_q ? lp->df_q : "");
-			fprintf(fo, "%s", lp->df_H);
+			VSB_cat(os, lp->df_U ? lp->df_U : "-");
+			VSB_cat(os, lp->df_q ? lp->df_q : "");
+			VSB_putc(os, ' ');
+			VSB_cat(os, lp->df_H ? lp->df_H : "HTTP/1.0");
 			break;
 
 		case 's':
 			/* %s */
-			fprintf(fo, "%s", lp->df_s ? lp->df_s : "");
+			VSB_cat(os, lp->df_s ? lp->df_s : "");
 			break;
 
 		case 't':
 			/* %t */
 			strftime(tbuf, sizeof tbuf, "[%d/%b/%Y:%T %z]", &lp->df_t);
-			fprintf(fo, "%s", tbuf);
+			VSB_cat(os, tbuf);
 			break;
 
 		case 'U':
-			fprintf(fo, "%s", lp->df_U);
+			VSB_cat(os, lp->df_U ? lp->df_U : "-");
 			break;
 
 		case 'u':
@@ -616,68 +670,75 @@ h_ncsa(void *priv, enum VSL_tag_e tag, unsigned fd,
 				q = strchr(rubuf, ':');
 				if (q != NULL)
 					*q = '\0';
-				fprintf(fo, "%s", rubuf);
+				VSB_cat(os, rubuf);
 				free(rubuf);
 			} else {
-				fprintf(fo, "-");
+				VSB_putc(os, '-');
 			}
 			break;
 
-		case '{':
-			if (strncmp(p, "{Referer}i", 10) == 0) {
-				fprintf(fo, "%s",
-					lp->df_Referer ? lp->df_Referer : "-");
-				p += 9;
+		case '{': {
+			const char *h, *tmp;
+			char fname[100], type;
+			tmp = p;
+			type = 0;
+			while (*tmp != '\0' && *tmp != '}')
+				tmp++;
+			if (*tmp == '}') {
+				tmp++;
+				type = *tmp;
+				memcpy(fname, p+1, tmp-p-2);
+				fname[tmp-p-2] = 0;
+			}
+
+			switch (type) {
+			case 'i':
+				h = req_header(lp, fname);
+				VSB_cat(os, h ? h : "-");
+				p = tmp;
 				break;
-			} else if (strncmp(p, "{Host}i", 7) == 0) {
-				fprintf(fo, "%s",
-					lp->df_Host ? lp->df_Host : "-");
-				p += 6;
+			case 'o':
+				h = resp_header(lp, fname);
+				VSB_cat(os, h ? h : "-");
+				p = tmp;
 				break;
-			} else if (strncmp(p, "{X-Forwarded-For}i", 18) == 0) {
-				/* %{Referer}i */
-				fprintf(fo, "%s",
-					lp->df_X_Forwarded_For ? lp->df_X_Forwarded_For : "-");
-				p += 17;
-				break;
-			} else if (strncmp(p, "{User-agent}i", 13) == 0) {
-				/* %{User-agent}i */
-				fprintf(fo, "%s",
-					lp->df_User_agent ? lp->df_User_agent : "-");
-				p += 12;
-				break;
-			} else if (strncmp(p, "{Varnish:", 9) == 0) {
-				/* Scan for what we're looking for */
-				const char *what = p+9;
-				if (strncmp(what, "time_firstbyte}x", 16) == 0) {
-					fprintf(fo, "%s", lp->df_ttfb);
-					p += 9+15;
+			case 'x':
+				if (strcmp(fname, "Varnish:time_firstbyte") == 0) {
+					VSB_cat(os, lp->df_ttfb);
+					p = tmp;
 					break;
-				} else if (strncmp(what, "hitmiss}x", 9) == 0) {
-					fprintf(fo, "%s", lp->df_hitmiss);
-					p += 9+8;
+				} else if (strcmp(fname, "Varnish:hitmiss") == 0) {
+					VSB_cat(os, (lp->df_hitmiss ? lp->df_hitmiss : "-"));
+					p = tmp;
 					break;
-				} else if (strncmp(what, "handling}x", 10) == 0) {
-					fprintf(fo, "%s", lp->df_handling);
-					p += 9+9;
+				} else if (strcmp(fname, "Varnish:handling") == 0) {
+					VSB_cat(os, (lp->df_handling ? lp->df_handling : "-"));
+					p = tmp;
 					break;
 				}
+			default:
+				fprintf(stderr, "Unknown format starting at: %s\n", --p);
+				exit(1);
 			}
+			break;
+		}
 			/* Fall through if we haven't handled something */
 			/* FALLTHROUGH*/
 		default:
-			fprintf(stderr, "Unknown format character: %c\n", *p);
+			fprintf(stderr, "Unknown format starting at: %s\n", --p);
 			exit(1);
 		}
 	}
-	fprintf(fo, "\n");
+	VSB_putc(os, '\n');
 
 	/* flush the stream */
+	VSB_finish(os);
+	fprintf(fo, "%s", VSB_data(os));
 	fflush(fo);
 
 	/* clean up */
 	clean_logline(lp);
-
+	VSB_delete(os);
 	return (reopen);
 }
 
