@@ -54,7 +54,7 @@ struct http {
 	unsigned		magic;
 #define HTTP_MAGIC		0x2f02169c
 	int			fd;
-	int			sfd;
+	int			*sfd;
 	int			timeout;
 	struct vtclog		*vl;
 
@@ -77,14 +77,14 @@ struct http {
 
 #define ONLY_CLIENT(hp, av)						\
 	do {								\
-		if (hp->sfd >= 0)					\
+		if (hp->sfd != NULL)					\
 			vtc_log(hp->vl, 0,				\
 			    "\"%s\" only possible in client", av[0]);	\
 	} while (0)
 
 #define ONLY_SERVER(hp, av)						\
 	do {								\
-		if (hp->sfd < 0)					\
+		if (hp->sfd == NULL)					\
 			vtc_log(hp->vl, 0,				\
 			    "\"%s\" only possible in server", av[0]);	\
 	} while (0)
@@ -107,7 +107,7 @@ synth_body(const char *len, int rnd)
 	AN(len);
 	i = strtoul(len, NULL, 0);
 	assert(i > 0);
-	b = malloc(i + 1);
+	b = malloc(i + 1L);
 	AN(b);
 	l = k = '!';
 	for (j = 0; j < i; j++) {
@@ -143,7 +143,8 @@ http_write(const struct http *hp, int lvl, const char *pfx)
 	vtc_dump(hp->vl, lvl, pfx, VSB_data(hp->vsb), VSB_len(hp->vsb));
 	l = write(hp->fd, VSB_data(hp->vsb), VSB_len(hp->vsb));
 	if (l != VSB_len(hp->vsb))
-		vtc_log(hp->vl, 0, "Write failed: %s", strerror(errno));
+		vtc_log(hp->vl, 0, "Write failed: (%d vs %d) %s",
+		    l, VSB_len(hp->vsb), strerror(errno));
 }
 
 /**********************************************************************
@@ -333,7 +334,7 @@ http_splitheader(struct http *hp, int req)
  */
 
 static int
-http_rxchar_eof(struct http *hp, int n)
+http_rxchar(struct http *hp, int n, int eof)
 {
 	int i;
 	struct pollfd pfd[1];
@@ -343,17 +344,27 @@ http_rxchar_eof(struct http *hp, int n)
 		pfd[0].events = POLLIN;
 		pfd[0].revents = 0;
 		i = poll(pfd, 1, hp->timeout);
-		if (i <= 0)
-			vtc_log(hp->vl, 0, "HTTP rx failed (poll: %s)",
-			    strerror(errno));
+		if (i == 0)
+			vtc_log(hp->vl, 0, "HTTP rx timeout (fd:%d %u ms)",
+			    hp->fd, hp->timeout);
+		if (i < 0)
+			vtc_log(hp->vl, 0, "HTTP rx failed (fd:%d poll: %s)",
+			    hp->fd, strerror(errno));
 		assert(i > 0);
 		assert(hp->prxbuf + n < hp->nrxbuf);
 		i = read(hp->fd, hp->rxbuf + hp->prxbuf, n);
-		if (i == 0)
+		if (!(pfd[0].revents & POLLIN))
+			vtc_log(hp->vl, 4,
+			    "HTTP rx poll (fd:%d revents: %x n=%d, i=%d)",
+			    hp->fd, pfd[0].revents, n, i);
+		if (i == 0 && eof)
 			return (i);
-		if (i <= 0)
-			vtc_log(hp->vl, 0, "HTTP rx failed (read: %s)",
-			    strerror(errno));
+		if (i == 0)
+			vtc_log(hp->vl, 0, "HTTP rx EOF (fd:%d read: %s)",
+			    hp->fd, strerror(errno));
+		if (i < 0)
+			vtc_log(hp->vl, 0, "HTTP rx failed (fd:%d read: %s)",
+			    hp->fd, strerror(errno));
 		hp->prxbuf += i;
 		hp->rxbuf[hp->prxbuf] = '\0';
 		n -= i;
@@ -361,17 +372,6 @@ http_rxchar_eof(struct http *hp, int n)
 	return (1);
 }
 
-static void
-http_rxchar(struct http *hp, int n)
-{
-	int i;
-
-	i = http_rxchar_eof(hp, n);
-	if (i <= 0)
-		vtc_log(hp->vl, 0, "HTTP rx failed (%s)", strerror(errno));
-	assert(i > 0);
-}
-
 static int
 http_rxchunk(struct http *hp)
 {
@@ -380,7 +380,7 @@ http_rxchunk(struct http *hp)
 
 	l = hp->prxbuf;
 	do
-		http_rxchar(hp, 1);
+		(void)http_rxchar(hp, 1, 0);
 	while (hp->rxbuf[hp->prxbuf - 1] != '\n');
 	vtc_dump(hp->vl, 4, "len", hp->rxbuf + l, -1);
 	i = strtoul(hp->rxbuf + l, &q, 16);
@@ -394,12 +394,12 @@ http_rxchunk(struct http *hp)
 	assert(*q == '\0' || vct_islws(*q));
 	hp->prxbuf = l;
 	if (i > 0) {
-		http_rxchar(hp, i);
+		(void)http_rxchar(hp, i, 0);
 		vtc_dump(hp->vl, 4, "chunk",
 		    hp->rxbuf + l, i);
 	}
 	l = hp->prxbuf;
-	http_rxchar(hp, 2);
+	(void)http_rxchar(hp, 2, 0);
 	if(!vct_iscrlf(hp->rxbuf[l]))
 		vtc_log(hp->vl, 0,
 		    "Wrong chunk tail[0] = %02x",
@@ -427,7 +427,7 @@ http_swallow_body(struct http *hp, char * const *hh, int body)
 	p = http_find_header(hh, "content-length");
 	if (p != NULL) {
 		l = strtoul(p, NULL, 0);
-		http_rxchar(hp, l);
+		(void)http_rxchar(hp, l, 0);
 		vtc_dump(hp->vl, 4, "body", hp->body, l);
 		hp->bodyl = l;
 		sprintf(hp->bodylen, "%d", l);
@@ -446,7 +446,7 @@ http_swallow_body(struct http *hp, char * const *hh, int body)
 	if (body) {
 		hp->body = hp->rxbuf + hp->prxbuf;
 		do  {
-			i = http_rxchar_eof(hp, 1);
+			i = http_rxchar(hp, 1, 1);
 			ll += i;
 		} while (i > 0);
 		vtc_dump(hp->vl, 4, "rxeof", hp->body, ll);
@@ -469,7 +469,7 @@ http_rxhdr(struct http *hp)
 	hp->prxbuf = 0;
 	hp->body = NULL;
 	while (1) {
-		http_rxchar(hp, 1);
+		(void)http_rxchar(hp, 1, 0);
 		p = hp->rxbuf + hp->prxbuf - 1;
 		for (i = 0; p > hp->rxbuf; p--) {
 			if (*p != '\n')
@@ -527,7 +527,7 @@ cmd_http_rxresp(CMD_ARGS)
 
 #define TRUST_ME(ptr)   ((void*)(uintptr_t)(ptr))
 
-#define OVERHEAD 64
+#define OVERHEAD 64L
 
 
 static void
@@ -586,7 +586,7 @@ cmd_http_gunzip_body(CMD_ARGS)
  */
 
 static void
-gzip_body(struct http *hp, const char *txt, char **body, int *bodylen)
+gzip_body(const struct http *hp, const char *txt, char **body, int *bodylen)
 {
 	int l, i;
 	z_stream vz;
@@ -899,7 +899,49 @@ cmd_http_send(CMD_ARGS)
 	AZ(av[2]);
 	vtc_dump(hp->vl, 4, "send", av[1], -1);
 	i = write(hp->fd, av[1], strlen(av[1]));
-	assert(i == strlen(av[1]));
+	if (i != strlen(av[1]))
+		vtc_log(hp->vl, 0, "Write error in http_send(): %s",
+		    strerror(errno));
+}
+
+/**********************************************************************
+ * Send a hex string
+ */
+
+static void
+cmd_http_sendhex(CMD_ARGS)
+{
+	struct http *hp;
+	char buf[3], *q;
+	uint8_t *p;
+	int i, j, l;
+
+	(void)cmd;
+	(void)vl;
+	CAST_OBJ_NOTNULL(hp, priv, HTTP_MAGIC);
+	AN(av[1]);
+	AZ(av[2]);
+	l = strlen(av[1]) / 2;
+	p = malloc(l);
+	AN(p);
+	q = av[1];
+	for (i = 0; i < l; i++) {
+		while (vct_issp(*q))
+			q++;
+		if (*q == '\0')
+			break;
+		memcpy(buf, q, 2);
+		q += 2;
+		buf[2] = '\0';
+		if (!vct_ishex(buf[0]) || !vct_ishex(buf[1]))
+			vtc_log(hp->vl, 0, "Illegal Hex char \"%c%c\"",
+			    buf[0], buf[1]);
+		p[i] = (uint8_t)strtoul(buf, NULL, 16);
+	}
+	vtc_hexdump(hp->vl, 4, "sendhex", (void*)p, i);
+	j = write(hp->fd, p, i);
+	assert(j == i);
+	free(p);
 
 }
 
@@ -991,7 +1033,7 @@ cmd_http_expect_close(CMD_ARGS)
 	(void)vl;
 	CAST_OBJ_NOTNULL(hp, priv, HTTP_MAGIC);
 	AZ(av[1]);
-	assert(hp->sfd >= 0);
+	assert(hp->sfd != NULL);
 
 	vtc_log(vl, 4, "Expecting close (fd = %d)", hp->fd);
 	while (1) {
@@ -1016,6 +1058,25 @@ cmd_http_expect_close(CMD_ARGS)
 	vtc_log(vl, 4, "fd=%d EOF, as expected", hp->fd);
 }
 
+/**********************************************************************
+ * close a new connection  (server only)
+ */
+
+static void
+cmd_http_close(CMD_ARGS)
+{
+	struct http *hp;
+
+	(void)cmd;
+	(void)vl;
+	CAST_OBJ_NOTNULL(hp, priv, HTTP_MAGIC);
+	AZ(av[1]);
+	assert(hp->sfd != NULL);
+	assert(*hp->sfd >= 0);
+	VTCP_close(&hp->fd);
+	vtc_log(vl, 4, "Closed");
+}
+
 /**********************************************************************
  * close and accept a new connection  (server only)
  */
@@ -1029,10 +1090,12 @@ cmd_http_accept(CMD_ARGS)
 	(void)vl;
 	CAST_OBJ_NOTNULL(hp, priv, HTTP_MAGIC);
 	AZ(av[1]);
-	assert(hp->sfd >= 0);
-	VTCP_close(&hp->fd);
+	assert(hp->sfd != NULL);
+	assert(*hp->sfd >= 0);
+	if (hp->fd >= 0)
+		VTCP_close(&hp->fd);
 	vtc_log(vl, 4, "Accepting");
-	hp->fd = accept(hp->sfd, NULL, NULL);
+	hp->fd = accept(*hp->sfd, NULL, NULL);
 	if (hp->fd < 0)
 		vtc_log(vl, 0, "Accepted failed: %s", strerror(errno));
 	vtc_log(vl, 3, "Accepted socket fd is %d", hp->fd);
@@ -1080,28 +1143,31 @@ static const struct cmds http_cmds[] = {
 	{ "gunzip",		cmd_http_gunzip_body },
 	{ "expect",		cmd_http_expect },
 	{ "send",		cmd_http_send },
+	{ "sendhex",		cmd_http_sendhex },
 	{ "chunked",		cmd_http_chunked },
 	{ "chunkedlen",		cmd_http_chunkedlen },
 	{ "delay",		cmd_delay },
 	{ "sema",		cmd_sema },
 	{ "expect_close",	cmd_http_expect_close },
+	{ "close",		cmd_http_close },
 	{ "accept",		cmd_http_accept },
 	{ "loop",		cmd_http_loop },
 	{ NULL,			NULL }
 };
 
-void
-http_process(struct vtclog *vl, const char *spec, int sock, int sfd)
+int
+http_process(struct vtclog *vl, const char *spec, int sock, int *sfd)
 {
 	struct http *hp;
 	char *s, *q;
+	int retval;
 
 	(void)sfd;
 	ALLOC_OBJ(hp, HTTP_MAGIC);
 	AN(hp);
 	hp->fd = sock;
-	hp->timeout = 5000;
-	hp->nrxbuf = 640*1024;
+	hp->timeout = 15000;
+	hp->nrxbuf = 2048*1024;
 	hp->vsb = VSB_new_auto();
 	hp->rxbuf = malloc(hp->nrxbuf);		/* XXX */
 	hp->sfd = sfd;
@@ -1116,9 +1182,11 @@ http_process(struct vtclog *vl, const char *spec, int sock, int sfd)
 	assert(q > s);
 	AN(s);
 	parse_string(s, http_cmds, hp, vl);
+	retval = hp->fd;
 	VSB_delete(hp->vsb);
 	free(hp->rxbuf);
 	free(hp);
+	return (retval);
 }
 
 /**********************************************************************
