@@ -227,8 +227,10 @@ vtc_hexdump(struct vtclog *vl, unsigned lvl, const char *pfx, const unsigned cha
 {
 	int nl = 1;
 	unsigned l;
+	double tx;
 
 	CHECK_OBJ_NOTNULL(vl, VTCLOG_MAGIC);
+	tx = TIM_mono() - t0;
 	assert(len >= 0);
 	assert(lvl < NLEAD);
 	AZ(pthread_mutex_lock(&vl->mtx));
@@ -236,8 +238,8 @@ vtc_hexdump(struct vtclog *vl, unsigned lvl, const char *pfx, const unsigned cha
 	if (pfx == NULL)
 		pfx = "";
 	if (str == NULL)
-		VSB_printf(vl->vsb, "%s %-4s %s(null)\n",
-		    lead[lvl], vl->id, pfx);
+		VSB_printf(vl->vsb, "%s %-4s %4.1f %s| (null)",
+		    lead[lvl], vl->id, tx, pfx);
 	else {
 		for (l = 0; l < len; l++, str++) {
 			if (l > 512) {
@@ -245,8 +247,8 @@ vtc_hexdump(struct vtclog *vl, unsigned lvl, const char *pfx, const unsigned cha
 				break;
 			}
 			if (nl) {
-				VSB_printf(vl->vsb, "%s %-4s %s| ",
-				    lead[lvl], vl->id, pfx);
+				VSB_printf(vl->vsb, "%s %-4s %4.1f %s| ",
+				    lead[lvl], vl->id, tx, pfx);
 				nl = 0;
 			}
 			VSB_printf(vl->vsb, " %02x", *str);
