@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006-2009 Linpro AS
+ * Copyright (c) 2006-2011 Varnish Software AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -26,8 +26,6 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: cli_priv.h 4477 2010-01-21 10:26:39Z phk $
- *
  * Varnish process internal CLI stuff.
  *
  * XXX: at a latter date we may want to move some to cli.h/libvarnishapi
@@ -54,6 +52,6 @@ struct cli_proto {
 };
 
 /* The implementation must provide these functions */
-void cli_out(struct cli *cli, const char *fmt, ...);
-void cli_quote(struct cli *cli, const char *str);
-void cli_result(struct cli *cli, unsigned r);
+void VCLI_Out(struct cli *cli, const char *fmt, ...);
+void VCLI_Quote(struct cli *cli, const char *str);
+void VCLI_SetResult(struct cli *cli, unsigned r);
