@@ -31,6 +31,9 @@
 #ifndef FLOPEN_H_INCLUDED
 #define FLOPEN_H_INCLUDED
 
+#include <sys/types.h>
+
 int flopen(const char *, int, ...);
+int fltest(int fd, pid_t *pid);
 
 #endif
