@@ -188,22 +188,26 @@ vsm_open(struct VSM_data *vd, int diag)
 		if (diag)
 			vd->diag(vd->priv, "Cannot mmap %s: %s\n",
 			    vd->fname, strerror(errno));
+		AZ(close(vd->vsm_fd));
+		vd->vsm_fd = -1;
+		vd->VSM_head = NULL;
 		return (1);
 	}
 	vd->vsm_end = (uint8_t *)vd->VSM_head + slh.shm_size;
 
-	for (j = 0; j < 20 && slh.alloc_seq == 0; j++)
+	for (j = 0; j < 20 && vd->VSM_head->alloc_seq == 0; j++)
 		(void)usleep(50000);
-	if (slh.alloc_seq == 0) {
+	if (vd->VSM_head->alloc_seq == 0) {
 		if (diag)
 			vd->diag(vd->priv, "File not initialized %s\n",
 			    vd->fname);
 		assert(0 == munmap((void*)vd->VSM_head, slh.shm_size));
 		AZ(close(vd->vsm_fd));
 		vd->vsm_fd = -1;
+		vd->VSM_head = NULL;
 		return (1);
 	}
-	vd->alloc_seq = slh.alloc_seq;
+	vd->alloc_seq = vd->VSM_head->alloc_seq;
 
 	if (vd->vsl != NULL)
 		VSL_Open_CallBack(vd);
@@ -220,7 +224,7 @@ VSM_Open(struct VSM_data *vd, int diag)
 	CHECK_OBJ_NOTNULL(vd, VSM_MAGIC);
 	AZ(vd->VSM_head);
 	if (!vd->n_opt)
-		VSM_n_Arg(vd, "");
+		(void)VSM_n_Arg(vd, "");
 	return (vsm_open(vd, diag));
 }
 
@@ -327,7 +331,7 @@ VSM_iter0(struct VSM_data *vd)
 	CHECK_OBJ_NOTNULL(vd, VSM_MAGIC);
 	vd->alloc_seq = vd->VSM_head->alloc_seq;
 	while (vd->alloc_seq == 0) {
-		usleep(50000);
+		(void)usleep(50000);
 		vd->alloc_seq = vd->VSM_head->alloc_seq;
 	}
 	CHECK_OBJ_NOTNULL(&vd->VSM_head->head, VSM_CHUNK_MAGIC);
@@ -354,7 +358,7 @@ VSM_itern(const struct VSM_data *vd, struct VSM_chunk **pp)
 
 /*--------------------------------------------------------------------*/
 unsigned
-VSM_Seq(struct VSM_data *vd)
+VSM_Seq(const struct VSM_data *vd)
 {
 
 	CHECK_OBJ_NOTNULL(vd, VSM_MAGIC);
