@@ -454,7 +454,7 @@ vcc_Acl_Hack(struct vcc *tl, char *b)
 	VTAILQ_INIT(&tl->acl);
 	tcond = tl->t->tok;
 	vcc_NextToken(tl);
-	bprintf(acln, "%u", tl->cnt);
+	bprintf(acln, "%u", tl->unique++);
 	vcc_acl_entry(tl);
 	vcc_acl_emit(tl, acln, 1);
 	sprintf(b, "%smatch_acl_anon_%s(sp, \v1)",
