@@ -145,7 +145,7 @@ vcc_NumVal(struct vcc *tl, double *d, int *frac)
 	vcc_NextToken(tl);
 }
 
-double
+static double
 vcc_DoubleVal(struct vcc *tl)
 {
 	double d;
@@ -454,7 +454,7 @@ vcc_Eval_Regsub(struct vcc *tl, struct expr **e, const struct symbol *sym)
 	SkipToken(tl, '(');
 
 	vcc_expr0(tl, &e2, STRING);
-	if (e2->fmt != STRING) 
+	if (e2->fmt != STRING)
 		vcc_expr_tostring(&e2, STRING);
 
 	SkipToken(tl, ',');
@@ -467,7 +467,7 @@ vcc_Eval_Regsub(struct vcc *tl, struct expr **e, const struct symbol *sym)
 
 	SkipToken(tl, ',');
 	vcc_expr0(tl, &e2, STRING);
-	if (e2->fmt != STRING) 
+	if (e2->fmt != STRING)
 		vcc_expr_tostring(&e2, STRING);
 	*e = vcc_expr_edit(STRING, "\v1, \v2)", *e, e2);
 	SkipToken(tl, ')');
@@ -886,6 +886,7 @@ static const struct cmps {
 	NUM_REL(INT),
 	NUM_REL(DURATION),
 	NUM_REL(BYTES),
+	NUM_REL(REAL),
 
 	{STRING,	T_EQ,	"!VRT_strcmp(\v1, \v2)" },
 	{STRING,	T_NEQ,	"VRT_strcmp(\v1, \v2)" },
